@@ -1,5 +1,5 @@
 /*
-   Copyright 1999-2018,2019 Thomas E. Dickey
+   Copyright 1999-2018,2019, Thomas E. Dickey
 
    Copyright (c) 1994, Joseph Arceneaux.  All rights reserved.
 
@@ -65,6 +65,7 @@ int indent_eqls;
 int indent_eqls_1st;
 
 int suppress_blanklines = 0;
+static int suppress_formatting = 0;
 static int not_first_line;
 
 int paren_target;
@@ -105,7 +106,32 @@ static void
 pass_char (int ch)
 {
   if (ch == '\n')
-    ++out_line_no;
+    {
+      ++out_line_no;
+      out_column_no = 0;
+    }
+  else if (ch == TAB)
+    {
+      int target_column = out_column_no + tabsize - (out_column_no % tabsize);
+      if (use_tabs || suppress_formatting)
+	{
+	  out_column_no = target_column;
+	}
+      else
+	{
+	  ch = ' ';
+	  while (out_column_no < target_column - 1)
+	    {
+	      putc (ch, output);
+	      ++out_column_no;
+	    }
+	  ++out_column_no;
+	}
+    }
+  else
+    {
+      ++out_column_no;
+    }
   putc (ch, output);
 }
 
@@ -116,6 +142,9 @@ pass_text (const char *s)
     pass_char (*s++);
 }
 
+#define at_current_eof(p) \
+	((unsigned) ((p) - current_input->data) == current_input->size)
+
 static char *
 pass_line (char *p)
 {
@@ -129,8 +158,7 @@ pass_line (char *p)
       in_prog_pos = p;
       cur_line = p;
     }
-  else if (*p == '\0'
-	   && ((unsigned int) (p - current_input->data) == current_input->size))
+  else if (*p == '\0' && at_current_eof (p))
     {
       buf_ptr = buf_end = in_prog_pos = p;
       had_eof = true;
@@ -245,28 +273,27 @@ pad_output (
 	     int my_current_col,
 	     int target_column)
 {
-  if (my_current_col >= target_column)
-    return my_current_col;
-
-  if (tabsize > 1)
+  if (my_current_col < target_column)
     {
-      int offset;
-
-      offset = tabsize - (my_current_col - 1) % tabsize;
-      while (my_current_col + offset <= target_column)
+      if (tabsize > 1)
 	{
-	  pass_char (TAB);
-	  my_current_col += offset;
-	  offset = tabsize;
+	  int offset;
+
+	  offset = tabsize - (my_current_col - 1) % tabsize;
+	  while (my_current_col + offset <= target_column)
+	    {
+	      pass_char (TAB);
+	      my_current_col += offset;
+	      offset = tabsize;
+	    }
+	}
+
+      while (my_current_col < target_column)
+	{
+	  pass_char (' ');
+	  my_current_col++;
 	}
     }
-
-  while (my_current_col < target_column)
-    {
-      pass_char (' ');
-      my_current_col++;
-    }
-
   return my_current_col;
 }
 
@@ -341,7 +368,6 @@ dump_line (void)
       if (e_lab != s_lab)
 	{			/* print lab, if any */
 	  int label_target = compute_label_target ();
-	  int pad_preproc = 0;
 	  char *skip_pound = s_lab;
 	  char *s_key;
 
@@ -364,8 +390,9 @@ dump_line (void)
 		{
 		  int adj = (!strncmp (s_key, "if", (size_t) 2) ||
 			     !strncmp (s_key, "el", (size_t) 2)) ? 1 : 0;
-		  pad_preproc = ((preprocessor_level (parser_state_tos) - adj)
-				 * preprocessor_indentation);
+		  int pad_preproc = ((preprocessor_level (parser_state_tos)
+				      - adj)
+				     * preprocessor_indentation);
 		  cur_col = pad_output (cur_col, cur_col + pad_preproc);
 		}
 	    }
@@ -601,17 +628,23 @@ int
 compute_code_target (void)
 {
   int target_col = parser_state_tos->ind_level + 1;
+  int rc;
 
   if (!parser_state_tos->paren_level)
     {
       if (parser_state_tos->ind_stmt)
 	target_col += continuation_indent;
-      return target_col;
+      rc = target_col;
     }
-
-  if (!lineup_to_parens)
-    return target_col + (continuation_indent * parser_state_tos->paren_level);
-  return paren_target;
+  else if (lineup_to_parens)
+    {
+      rc = paren_target;
+    }
+  else
+    {
+      rc = target_col + (continuation_indent * parser_state_tos->paren_level);
+    }
+  return rc;
 }
 
 int
@@ -739,7 +772,7 @@ unterminated_inhibit (int start_no)
 {
   int save_line_no = in_line_no;
   in_line_no = start_no + 1;
-  message (1, "unterminated INDENT-OFF comment", 0);
+  message (1, "unterminated INDENT-OFF comment");
   in_line_no = save_line_no;
 }
 
@@ -874,6 +907,7 @@ fill_buffer (void)
 	      int inhibited = 1;
 	      int starting_no = in_line_no;
 
+	      suppress_formatting = 1;
 	      if (s_com != e_com || s_lab != e_lab || s_code != e_code)
 		dump_line ();
 	      while (q < p)
@@ -883,8 +917,7 @@ fill_buffer (void)
 		{
 		  while (*p != '\0' && *p != EOL)
 		    pass_char (*p++);
-		  if (*p == '\0'
-		      && ((unsigned int) (p - current_input->data) == current_input->size))
+		  if (*p == '\0' && at_current_eof (p))
 		    {
 		      buf_ptr = buf_end = in_prog_pos = p;
 		      had_eof = true;
@@ -915,10 +948,7 @@ fill_buffer (void)
 			    {
 			      while (*p != '\0' && *p != EOL)
 				pass_char (*p++);
-			      if (*p == '\0'
-				  && ((unsigned int) (p -
-						      current_input->data)
-				      == current_input->size))
+			      if (*p == '\0' && at_current_eof (p))
 				{
 				  buf_ptr = buf_end = in_prog_pos = p;
 				  had_eof = true;
@@ -930,9 +960,13 @@ fill_buffer (void)
 				    {
 				      ++in_line_no;
 				      inhibited = 0;
+				      suppress_formatting = 0;
 				      cur_line = p + 1;
-				      if (*cur_line == '\0')
-					pass_char (*p++);
+				      if (*cur_line == '\0' &&
+					  at_current_eof (cur_line))
+					{
+					  pass_char (EOL);
+					}
 				    }
 				  else
 				    pass_char (*p++);
@@ -947,7 +981,7 @@ fill_buffer (void)
 			}
 		      else if (!strncmp (p, "*INDENT-", (size_t) 8))
 			{
-			  message (1, "unexpected INDENT-comment", 0);
+			  message (1, "unexpected INDENT-comment");
 			}
 		    }
 		}
@@ -970,10 +1004,10 @@ fill_buffer (void)
 	  in_prog_pos = p + 1;
 	}
       /* Here for embedded NULLs */
-      else if ((unsigned int) (p - current_input->data) < current_input->size)
+      else if ((unsigned) (p - current_input->data) < current_input->size)
 	{
 	  message (-1, "Warning: File %s contains NULL-characters\n",
-		   current_input->name, 0);
+		   current_input->name);
 	  p++;
 	}
       /* Here for EOF with no terminating newline char. */
